package vn.teko.loyalty.loyalty_flutter

import androidx.annotation.NonNull
import io.flutter.embedding.engine.plugins.FlutterPlugin
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import vn.teko.loyalty.core.LoyaltyService
import vn.teko.loyalty.terra.TerraLoyalty
import vn.teko.terra.core.android.terra.TerraApp

/** LoyaltyFlutterPlugin */
class LoyaltyFlutterPlugin : FlutterPlugin, LoyaltyFlutter.FlutterLoyaltyApi {
    private val mainScope = CoroutineScope(Dispatchers.Main)

    override fun onAttachedToEngine(flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
        LoyaltyFlutter.FlutterLoyaltyApi.setup(flutterPluginBinding.binaryMessenger, this)
    }

    override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
        LoyaltyFlutter.FlutterLoyaltyApi.setup(binding.binaryMessenger, null)
    }

    override fun getInstance(appName: String): LoyaltyFlutter.CommonResult {
        val resultStatus = LoyaltyFlutter.CommonResult()
        try {
            getTerraLoyalty(appName)
            resultStatus.isSuccess = true
        } catch (error: Throwable) {
            resultStatus.isSuccess = false
            resultStatus.error = error.message
        }

        return resultStatus
    }

    override fun getConfiguration(
        appName: String,
        result: LoyaltyFlutter.Result<LoyaltyFlutter.FlutterLoyaltyConfigurationResult>?
    ) {
        val terraLoyalty = getTerraLoyalty(appName)
        val config = terraLoyalty.getConfiguration()
        result?.success(ModelConverter.convertLoyaltyConfiguration(config))
    }

    override fun registerMember(
        appName: String,
        request: LoyaltyFlutter.FlutterRegisterMemeberRequest,
        result: LoyaltyFlutter.Result<LoyaltyFlutter.FlutterRegisterMemberResult>?
    ) {
        val terraLoyalty = getTerraLoyalty(appName)
        mainScope.launch {
            val memberResult =
                terraLoyalty.registerMember(ModelConverter.convertLoyaltyRequest(request))
            val flutterResult: LoyaltyFlutter.FlutterRegisterMemberResult =
                if (memberResult.isFailure()) {
                    LoyaltyFlutter.FlutterRegisterMemberResult().apply {
                        isSuccess = false
                        error = memberResult.exception().message
                    }
                } else {
                    ModelConverter.convertRegisterMemberResult(memberResult.get())
                }

            result?.success(flutterResult)
        }
    }

    override fun getMembershipQr(
        appName: String,
        result: LoyaltyFlutter.Result<LoyaltyFlutter.FlutterMembershipQrResult>?
    ) {
        val terraLoyalty = getTerraLoyalty(appName)
        mainScope.launch {
            val memberResult = terraLoyalty.getMembershipQr()
            val flutterResult: LoyaltyFlutter.FlutterMembershipQrResult =
                if (memberResult.isFailure()) {
                    LoyaltyFlutter.FlutterMembershipQrResult().apply {
                        isSuccess = false
                        error = memberResult.exception().message
                    }
                } else {
                    ModelConverter.convertMembershipQrResult(memberResult.get())
                }

            result?.success(flutterResult)
        }
    }

    override fun getMemberInfo(
        appName: String,
        request: LoyaltyFlutter.FlutterMemberInfoRequest,
        result: LoyaltyFlutter.Result<LoyaltyFlutter.FlutterMemberInfoResult>?
    ) {
        val terraLoyalty = getTerraLoyalty(appName)
        mainScope.launch {
            val memberResult =
                terraLoyalty.getMemberInfo(ModelConverter.convertMemberInfoRequest(request))
            val flutterResult: LoyaltyFlutter.FlutterMemberInfoResult =
                if (memberResult.isFailure()) {
                    LoyaltyFlutter.FlutterMemberInfoResult().apply {
                        isSuccess = false
                        error = memberResult.exception().message
                    }
                } else {
                    ModelConverter.convertMemberInfoResult(memberResult.get())
                }

            result?.success(flutterResult)
        }
    }

    override fun getTransactionHistory(
        appName: String,
        request: LoyaltyFlutter.FlutterTransactionHistoryRequest,
        result: LoyaltyFlutter.Result<LoyaltyFlutter.FlutterTransactionHistoryResult>?
    ) {
        val terraLoyalty = getTerraLoyalty(appName)
        mainScope.launch {
            val memberResult =
                terraLoyalty.getTransactionHistory(
                    ModelConverter.convertTransactionHistoryRequest(
                        request
                    )
                )
            val flutterResult: LoyaltyFlutter.FlutterTransactionHistoryResult =
                if (memberResult.isFailure()) {
                    LoyaltyFlutter.FlutterTransactionHistoryResult().apply {
                        isSuccess = false
                        error = memberResult.exception().message
                    }
                } else {
                    ModelConverter.convertTransactionHistoryResult(memberResult.get())
                }

            result?.success(flutterResult)
        }
    }

    override fun getNetworkConfig(
        appName: String,
        result: LoyaltyFlutter.Result<LoyaltyFlutter.FlutterNetworkConfigResult>?
    ) {
        val terraLoyalty = getTerraLoyalty(appName)
        mainScope.launch {
            val memberResult =
                terraLoyalty.getNetworkConfig()
            val flutterResult: LoyaltyFlutter.FlutterNetworkConfigResult =
                if (memberResult.isFailure()) {
                    LoyaltyFlutter.FlutterNetworkConfigResult().apply {
                        isSuccess = false
                        error = memberResult.exception().message
                    }
                } else {
                    ModelConverter.convertNetworkConfig(memberResult.get())
                }

            result?.success(flutterResult)
        }
    }

    override fun getClientId(
        appName: String,
        result: LoyaltyFlutter.Result<LoyaltyFlutter.FlutterClientIdResult>?
    ) {
        val terraLoyalty = getTerraLoyalty(appName)
        result?.success(LoyaltyFlutter.FlutterClientIdResult().also {
            it.isSuccess = true
            it.clientId = terraLoyalty.getClientId()
        })
    }

    override fun assignMemberCardId(
        appName: String,
        memberId: String,
        memberCardId: String,
        result: LoyaltyFlutter.Result<LoyaltyFlutter.CommonResult>?
    ) {
        val terraLoyalty = getTerraLoyalty(appName)
        mainScope.launch {
            val memberResult =
                terraLoyalty.assignMemberCardId(memberId, memberCardId)
            val flutterResult: LoyaltyFlutter.CommonResult =
                if (memberResult.isFailure()) {
                    LoyaltyFlutter.CommonResult().apply {
                        isSuccess = false
                        error = memberResult.exception().message
                    }
                } else {
                    LoyaltyFlutter.CommonResult().apply {
                        isSuccess = true
                    }
                }

            result?.success(flutterResult)
        }
    }

    companion object {
        private fun getTerraLoyalty(appName: String): LoyaltyService {
            return TerraLoyalty.getInstance(TerraApp.getInstance(appName))
        }
    }
}
