package vn.teko.loyalty.loyalty_flutter

import vn.teko.loyalty.core.MemberInfoRequestType
import vn.teko.loyalty.core.model.api.query.TransactionHistoryQuery
import vn.teko.loyalty.core.model.api.request.RegisterMemberRequest
import vn.teko.loyalty.core.model.api.response.ExtraPageData
import vn.teko.loyalty.core.model.configuration.LoyaltyConfiguration
import vn.teko.loyalty.core.model.member.EncryptedMemCode
import vn.teko.loyalty.core.model.member.MemberInfo
import vn.teko.loyalty.core.model.member.RegisterMemberResult
import vn.teko.loyalty.core.model.network.Network
import vn.teko.loyalty.core.model.network.NetworkConfig
import vn.teko.loyalty.core.model.network.NetworkConfigResult
import vn.teko.loyalty.core.model.network.Tier
import vn.teko.loyalty.core.model.transaction.Transaction
import vn.teko.loyalty.core.model.transaction.TransactionHistoryResult

object ModelConverter {
    fun convertLoyaltyConfiguration(config: LoyaltyConfiguration): LoyaltyFlutter.FlutterLoyaltyConfigurationResult {
        val result = LoyaltyFlutter.FlutterLoyaltyConfigurationResult().apply { isSuccess = true }
        val flutterConfig = LoyaltyFlutter.FlutterLoyaltyConfiguration().apply {
            linkMembershipUrl = config.linkMembershipUrl
            registerMemberUrl = config.registerMemberUrl
            allowLinkMembership = config.allowLinkMembership
            allowRegisterMembership = config.allowRegisterMembership
            membershipCodeFormat = config.membershipCodeFormat
        }

        result.loyaltyConfiguration = flutterConfig
        return result
    }

    fun convertLoyaltyRequest(request: LoyaltyFlutter.FlutterRegisterMemeberRequest): RegisterMemberRequest {
        val gender = when (request.gender) {
            LoyaltyFlutter.FlutterGender.female -> RegisterMemberRequest.Gender.FEMALE
            LoyaltyFlutter.FlutterGender.male -> RegisterMemberRequest.Gender.MALE
            LoyaltyFlutter.FlutterGender.none -> RegisterMemberRequest.Gender.NONE
            else -> RegisterMemberRequest.Gender.NONE
        }

        return RegisterMemberRequest(
            phone = request.phone ?: "",
            name = request.name ?: "",
            address = request.address,
            dateOfBirth = request.dateOfBirth,
            email = request.email,
            gender = gender,
            idCardNumber = request.idCardNumber,
            idCitizenNumber = request.idCitizenNumber,
            passportNumber = request.passportNumber,
        )
    }

    fun convertRegisterMemberResult(registerMemberResult: RegisterMemberResult): LoyaltyFlutter.FlutterRegisterMemberResult {
        return LoyaltyFlutter.FlutterRegisterMemberResult().apply {
            isSuccess = true
            registerMember = LoyaltyFlutter.FlutterRegisterMember().also { registerMember ->
                registerMember.memberId = registerMemberResult.memberId
            }
        }
    }

    fun convertMembershipQrResult(encryptedMemCode: EncryptedMemCode): LoyaltyFlutter.FlutterMembershipQrResult {
        val membershipQr = LoyaltyFlutter.FlutterMembershipQr().apply {
            code = encryptedMemCode.code
            issueAt = encryptedMemCode.issueAt
            expireAt = encryptedMemCode.expireAt
        }

        return LoyaltyFlutter.FlutterMembershipQrResult().apply {
            this.isSuccess = true
            this.membershipQr = membershipQr
        }
    }

    fun convertMemberInfoRequest(request: LoyaltyFlutter.FlutterMemberInfoRequest): MemberInfoRequestType {
        if (request.phone.isNullOrBlank() && request.memberQrCode.isNullOrBlank()) {
            return MemberInfoRequestType.CurrentUser
        }

        if (!request.phone.isNullOrBlank()) {
            return MemberInfoRequestType.Phone(request.phone!!)
        }

        return MemberInfoRequestType.MemberQrCode(request.memberQrCode!!)
    }

    fun convertMemberInfoResult(memberInfo: MemberInfo): LoyaltyFlutter.FlutterMemberInfoResult {
        val flutterResult = LoyaltyFlutter.FlutterMemberInfoResult().apply { isSuccess = true }
        flutterResult.flutterMemberInfo = memberInfo.toFlutterMemberInfo()
        return flutterResult
    }

    fun convertTransactionHistoryRequest(request: LoyaltyFlutter.FlutterTransactionHistoryRequest): TransactionHistoryQuery {
        return TransactionHistoryQuery().apply {
            this.merchantCode = request.merchantCode
            this.fromTime = request.fromTime
            this.toTime = request.toTime
            this.txnTypes = request.txnTypes.toNativeTransactions()
            this.status = request.status.toNativeStatus()
            this.page = request.page?.toInt()
            this.pageSize = request.pageSize?.toInt()
        }
    }

    fun convertTransactionHistoryResult(transactionHistoryResult: TransactionHistoryResult): LoyaltyFlutter.FlutterTransactionHistoryResult {
        val flutterResult =
            LoyaltyFlutter.FlutterTransactionHistoryResult().apply { isSuccess = true }

        val flutterTransactions = transactionHistoryResult.transactions.toFlutterTransactions()
        val flutterPageData = transactionHistoryResult.pageData.toFlutterPageData()

        flutterResult.transactionHistory = LoyaltyFlutter.FlutterTransactionHistory().apply {
            this.transactions = flutterTransactions
            this.pageData = flutterPageData
        }

        return flutterResult
    }

    fun convertNetworkConfig(networkConfigResult: NetworkConfigResult): LoyaltyFlutter.FlutterNetworkConfigResult {
        val flutterResult = LoyaltyFlutter.FlutterNetworkConfigResult().apply { isSuccess = true }

        val flutterNetworkInfo = networkConfigResult.networkInfo.toFlutterNetwork()
        val flutterNetworkConfig = networkConfigResult.networkConfig.toFlutterNetworkConfig()
        val flutterNetworkTiers = networkConfigResult.networkTiers.toFlutterTiers()

        flutterResult.networkConfig = LoyaltyFlutter.FlutterNetworkData().also {
            it.networkInfo = flutterNetworkInfo
            it.networkTiers = flutterNetworkTiers
            it.networkConfig = flutterNetworkConfig
        }

        return flutterResult
    }

    private fun MemberInfo.toFlutterMemberInfo(): LoyaltyFlutter.FlutterMemberInfo {
        val flutterDefaultCardType = when (defaultCardType) {
            MemberInfo.CardType.MEMBER_CARD -> LoyaltyFlutter.FlutterCardType.memberCard
            else -> LoyaltyFlutter.FlutterCardType.qr
        }

        return LoyaltyFlutter.FlutterMemberInfo().also {
            it.memberQrCode = memberQrCode
            it.memberId = memberId
            it.phone = phone
            it.memberCardId = memberCardId
            it.name = name
            it.isActive = isActive
            it.point = point
            it.tierPoint = tierPoint
            it.keepTierUntil = keepTierUntil
            it.accumulationFrom = accumulationFrom
            it.accumulationTo = accumulationTo
            it.currentTierCode = currentTierCode
            it.currentTierName = currentTierName
            it.currentTierMinPoint = currentTierMinPoint?.toLong()
            it.nextTierName = nextTierName
            it.nextTierMinPoint = nextTierMinPoint?.toLong()
            it.defaultCardType = flutterDefaultCardType
        }

    }

    private fun Network.toFlutterNetwork(): LoyaltyFlutter.FlutterNetwork {
        return LoyaltyFlutter.FlutterNetwork().also {
            it.id = id
            it.name = name
        }
    }

    private fun NetworkConfig.toFlutterNetworkConfig(): LoyaltyFlutter.FlutterNetworkConfig {
        return LoyaltyFlutter.FlutterNetworkConfig().also { flutterNetworkConfig ->
            flutterNetworkConfig.exchangeRate = exchangeRate.toLong()
            flutterNetworkConfig.currencyImg = currencyImg
            flutterNetworkConfig.currencyName = currencyName
            flutterNetworkConfig.cardDefaultImg = cardDefaultImg
            flutterNetworkConfig.avatarDefaultImg = avatarDefaultImg
            flutterNetworkConfig.allowToGetMemberByPhone = allowToGetMemberByPhone
            flutterNetworkConfig.faqWebUrl = faqWebUrl
            flutterNetworkConfig.benefitWebUrl = benefitWebUrl
            flutterNetworkConfig.brandColor = brandColor
            flutterNetworkConfig.allowMerchantIssueMemberCard = allowMerchantIssueMemberCard
        }
    }

    private fun List<Tier>?.toFlutterTiers(): List<LoyaltyFlutter.FlutterTier>? {
        return this?.map { tier ->
            LoyaltyFlutter.FlutterTier().also {
                it.code = tier.code
                it.name = tier.name
                it.fromPoint = tier.fromPoint.toLong()
                it.cardTieredImg = tier.cardTieredImg
                it.tierIconImg = tier.tierIconImg
            }
        }
    }

    private fun Transaction.TransactionType.toFlutterType(): LoyaltyFlutter.FlutterTransactionType {
        return when (this) {
            Transaction.TransactionType.GRANT -> LoyaltyFlutter.FlutterTransactionType.grant
            Transaction.TransactionType.SPEND -> LoyaltyFlutter.FlutterTransactionType.spend
            Transaction.TransactionType.RETURN_GRANT -> LoyaltyFlutter.FlutterTransactionType.returnGrant
            Transaction.TransactionType.RETURN_SPEND -> LoyaltyFlutter.FlutterTransactionType.returnSpend
            Transaction.TransactionType.INITIATE -> LoyaltyFlutter.FlutterTransactionType.initiate
            Transaction.TransactionType.INITIATE_PROMOTION -> LoyaltyFlutter.FlutterTransactionType.initiatePromotion
            Transaction.TransactionType.ISSUE -> LoyaltyFlutter.FlutterTransactionType.issue
            else -> LoyaltyFlutter.FlutterTransactionType.grant
        }
    }

    private fun Transaction.Status.toFlutterStatus(): LoyaltyFlutter.FlutterStatus {
        return when (this) {
            Transaction.Status.INIT -> LoyaltyFlutter.FlutterStatus.init
            Transaction.Status.INIT_FAILURE -> LoyaltyFlutter.FlutterStatus.initFailure
            Transaction.Status.NO_ACCOUNTING -> LoyaltyFlutter.FlutterStatus.noAccouning
            Transaction.Status.ACCOUNTING_SUCCESS -> LoyaltyFlutter.FlutterStatus.accountingSuccess
            Transaction.Status.ACCOUNTING_FAILURE -> LoyaltyFlutter.FlutterStatus.accountingFailure
            Transaction.Status.ACCOUNTING -> LoyaltyFlutter.FlutterStatus.accouting
            else -> LoyaltyFlutter.FlutterStatus.accountingSuccess
        }
    }

    private fun LoyaltyFlutter.FlutterStatus?.toNativeStatus(): Transaction.Status? {
        return when (this) {
            LoyaltyFlutter.FlutterStatus.init -> Transaction.Status.INIT
            LoyaltyFlutter.FlutterStatus.initFailure -> Transaction.Status.INIT_FAILURE
            LoyaltyFlutter.FlutterStatus.noAccouning -> Transaction.Status.NO_ACCOUNTING
            LoyaltyFlutter.FlutterStatus.accountingSuccess -> Transaction.Status.ACCOUNTING_SUCCESS
            LoyaltyFlutter.FlutterStatus.accountingFailure -> Transaction.Status.ACCOUNTING_FAILURE
            LoyaltyFlutter.FlutterStatus.accouting -> Transaction.Status.ACCOUNTING
            else -> null
        }
    }

    private fun List<String>?.toNativeTransactions(): List<Transaction.TransactionType>? {
        val nativeTransactionType = Transaction.TransactionType.values();
        return this?.mapNotNull { txt ->
            nativeTransactionType.firstOrNull { it.value == txt }
        }
    }

    private fun ExtraPageData.toFlutterPageData(): LoyaltyFlutter.FlutterPageData {
        return LoyaltyFlutter.FlutterPageData().also { flutterPageData ->
            flutterPageData.currentPage = currentPage.toLong()
            flutterPageData.totalPage = totalPage.toLong()
            flutterPageData.pageSize = pageSize.toLong()
            flutterPageData.totalItems = totalItems.toLong()
        }
    }

    private fun List<Transaction>.toFlutterTransactions(): List<LoyaltyFlutter.FlutterTransaction> {
        return map { transaction ->
            val accountingType = when (transaction.accountingType) {
                Transaction.AccountingType.CREDIT -> LoyaltyFlutter.FlutterAccountingType.credit
                else -> LoyaltyFlutter.FlutterAccountingType.debit
            }

            LoyaltyFlutter.FlutterTransaction().apply {
                this.id = transaction.id
                this.refId = transaction.refId
                this.orderCode = transaction.orderCode
                this.txnType = transaction.txnType.toFlutterType()
                this.txnTypeText = transaction.txnTypeText
                this.txnDesc = transaction.txnDesc
                this.accountingType = accountingType
                this.status = transaction.status.toFlutterStatus()
                this.point = transaction.point.toLong()
                this.createdAt = transaction.createdAt
                this.createdBy = transaction.createdBy
                this.networkId = transaction.networkId
                this.networkName = transaction.networkName
                this.merchantCode = transaction.merchantCode
                this.merchantName = transaction.merchantName
                this.terminalCode = transaction.terminalCode
                this.terminalName = transaction.terminalName
                this.terminalAddress = transaction.terminalAddress
            }
        }
    }
}