//
//  ModelConverter.swift
//  loyalty_flutter
//
//  Created by Cao Trung on 22/06/2022.
//

import Foundation
import LoyaltyCore
import LoyaltyModel

func convertLoyaltyConfiguration(config: LoyaltyServiceConfig) -> FlutterLoyaltyConfigurationResult {
    let result = FlutterLoyaltyConfigurationResult()
    result.isSuccess = true
    
    let flutterConfig = FlutterLoyaltyConfiguration()
    flutterConfig.linkMembershipUrl = config.linkMembershipUrl
    flutterConfig.registerMemberUrl = config.registerMemberUrl
    flutterConfig.allowLinkMembership = convertBoolToNSNumber(config.allowLinkMembership)
    flutterConfig.allowRegisterMembership = convertBoolToNSNumber(config.allowRegisterMembership)
    flutterConfig.membershipCodeFormat = config.membershipCodeFormat.rawValue
    
    result.loyaltyConfiguration = flutterConfig
    
    return result
}

func convertLoyaltyRegisterMemberRequest(request: FlutterRegisterMemeberRequest) -> RegisterMemberRequestParam {
    return RegisterMemberRequestParam(
        name: request.name ?? "",
        phone: request.phone ?? "",
        email: request.email,
        address: request.address,
        dateOfBirth: request.dateOfBirth,
        gender: convertGender(flutterGender:  request.gender),
        idCardNumber: request.idCardNumber,
        idCitizenNumber: request.idCitizenNumber,
        passportNumber: request.passportNumber
    )
}

private func convertGender(flutterGender: FlutterGender) -> RegisterMemberRequestParam.Gender? {
    switch flutterGender {
    case FlutterGender.female:
        return RegisterMemberRequestParam.Gender.female
    case FlutterGender.male:
        return RegisterMemberRequestParam.Gender.male
    case FlutterGender.none:
        return RegisterMemberRequestParam.Gender.undefined
    default:
        return nil
    }
}

func convertRegisterMemberResult(registerMemberResult: RegisterMemberResult) -> FlutterRegisterMemberResult {
    let result = FlutterRegisterMemberResult()
    let registerMember = FlutterRegisterMember()
    registerMember.memberId = registerMemberResult.memberId
    
    result.isSuccess = true
    result.registerMember = registerMember
    
    return result
}

func convertMembershipQrResult(encryptedMemCode: QrData) -> FlutterMembershipQrResult {
    let membershipQr = FlutterMembershipQr()
    membershipQr.code = encryptedMemCode.code
    membershipQr.issueAt = NSNumber(value: encryptedMemCode.issueAt)
    membershipQr.expireAt = NSNumber(value: encryptedMemCode.expireAt)

    let flutterResult =  FlutterMembershipQrResult()
    flutterResult.isSuccess = true
    flutterResult.membershipQr = membershipQr
    
    return flutterResult
}

func convertMemberInfoType(request: FlutterMemberInfoRequest) -> MemberInfoRequestType {
    if (request.phone == nil && request.memberQrCode == nil) {
        return MemberInfoRequestType.currentUser
    }
    
    if (request.phone != nil) {
        return MemberInfoRequestType.phone(request.phone!)
    }
    
    return MemberInfoRequestType.memberQrCode(request.memberQrCode!)
}

func convertMemberInfoResult(memberInfo: MemberInfo) -> FlutterMemberInfoResult {
    let flutterResult = FlutterMemberInfoResult()
    flutterResult.isSuccess  = true
    
    let flutterMemberInfo = FlutterMemberInfo()
    flutterMemberInfo.memberQrCode = memberInfo.memberQrCode
    flutterMemberInfo.memberId = memberInfo.memberId
    flutterMemberInfo.phone = memberInfo.phone
    flutterMemberInfo.memberCardId = memberInfo.memberCardId
    flutterMemberInfo.name = memberInfo.name
    flutterMemberInfo.isActive = convertBoolToNSNumber(memberInfo.isActive)
    flutterMemberInfo.point = NSNumber(value: memberInfo.point)
    flutterMemberInfo.tierPoint = convertIntToNSNumber(memberInfo.tierPoint)
    flutterMemberInfo.keepTierUntil = convertIntToNSNumber(memberInfo.keepTierUntil)
    flutterMemberInfo.accumulationFrom = convertIntToNSNumber(memberInfo.accumulationFrom)
    flutterMemberInfo.accumulationTo = convertIntToNSNumber(memberInfo.accumulationTo)
    flutterMemberInfo.currentTierCode = memberInfo.currentTierCode
    flutterMemberInfo.currentTierName = memberInfo.currentTierName
    flutterMemberInfo.currentTierMinPoint = convertIntToNSNumber(memberInfo.currentTierMinPoint)
    flutterMemberInfo.nextTierName = memberInfo.nextTierName
    flutterMemberInfo.nextTierMinPoint = convertIntToNSNumber(memberInfo.nextTierMinPoint)
    flutterMemberInfo.defaultCardType = convertFlutterCardType(card: memberInfo.defaultCardType ?? LoyaltyCardType.qr)
    
    flutterResult.flutterMemberInfo = flutterMemberInfo
    return flutterResult
}

func convertTransactionHistoryRequeset(request: FlutterTransactionHistoryRequest) -> HistoryRequestParams {
    let txnTypes = request.txnTypes?.map({ strType in
        convertStringTransactionType(transactionType: strType)
    })
    
    let param = HistoryRequestParams(
        txnTypes: txnTypes ?? [],
        merchantCode: request.merchantCode,
        fromTime: request.fromTime,
        toTime: request.toTime,
        status: convertStatus(status: request.status),
        page: request.page?.intValue ?? 1,
        size: request.pageSize?.intValue ?? 20
    )
    
    return param
}

func convertTransactionHistoryResult(transactionHistoryResult: HistoryResponse) -> FlutterTransactionHistoryResult {
    let flutterResult = FlutterTransactionHistoryResult()
    guard let transactionHistory = transactionHistoryResult.result else {
        flutterResult.isSuccess = false
        flutterResult.error = "Loyalty Api Error"
        return flutterResult
    }
    
    let transactions = transactionHistory.transactions.toFlutterTransactions()
    let pageData = transactionHistoryResult.extra.toFlutterPaging()
    
    let flutterTransactionHistory = FlutterTransactionHistory()
    flutterTransactionHistory.transactions = transactions
    flutterTransactionHistory.pageData = pageData
    
    flutterResult.transactionHistory = flutterTransactionHistory
    flutterResult.isSuccess = true
    return flutterResult
}

private func convertAccoutingType(accounting: AccountingType) -> FlutterAccountingType {
    switch accounting {
    case AccountingType.Credit:
        return FlutterAccountingType.credit
    case AccountingType.Debit:
        return FlutterAccountingType.debit
    default:
        return FlutterAccountingType.debit
    }
}

private func convertStatus(status: TransactionStatus) -> FlutterStatus {
    switch status {
    case TransactionStatus.`init`:
        return FlutterStatus.`init`
    case TransactionStatus.failInit:
        return FlutterStatus.initFailure
    case TransactionStatus.unfinished:
        return FlutterStatus.noAccouning
    case TransactionStatus.finished:
        return FlutterStatus.accountingSuccess
    case TransactionStatus.fail:
        return FlutterStatus.accountingFailure
    case TransactionStatus.processing:
        return FlutterStatus.accouting
    default:
        return FlutterStatus.`init`
    }
}

func convertNetworkConfig(networkConfigResponse: NetworkConfigResponse) -> FlutterNetworkConfigResult {
    let flutterResult = FlutterNetworkConfigResult()
    
    guard let networkConfigResult = networkConfigResponse.result else {
        flutterResult.isSuccess = false
        flutterResult.error = "Loyalty Api Error"
        return flutterResult
    }
    
    flutterResult.isSuccess = true
    
    let flutterNetworkInfo = FlutterNetwork()
    flutterNetworkInfo.id = String(networkConfigResult.networkInfo?.id ?? 0)
    flutterNetworkInfo.name = networkConfigResult.networkInfo?.name
    
    let flutterNetworkTiers = networkConfigResult.networkTiers?.map { tier -> FlutterTier in
        let flutterTier = FlutterTier()
        flutterTier.code = tier.code
        flutterTier.name = tier.name
        flutterTier.fromPoint = convertIntToNSNumber(tier.fromPoint)
        flutterTier.cardTieredImg = tier.cardTieredImg
        flutterTier.tierIconImg = tier.cardTieredImg
        
        return flutterTier
    }
    
    let flutterNetworkData = FlutterNetworkData()
    flutterNetworkData.networkInfo = flutterNetworkInfo
    flutterNetworkData.networkTiers = flutterNetworkTiers
    flutterNetworkData.networkConfig = networkConfigResult.networkConfig != nil ? networkConfigResult.networkConfig?.toFlutterNetworkConfig() : nil
    
    flutterResult.networkConfig = flutterNetworkData
    return  flutterResult
}

private func convertIntToNSNumber(_ value: Int?) -> NSNumber? {
    guard let value = value else {
        return nil
    }
    
    return NSNumber(value: value)
}

private func convertBoolToNSNumber(_ value: Bool?) -> NSNumber? {
    guard let value = value else {
        return nil
    }
    
    return NSNumber(value: value)
}

extension Array where Element == Transaction {
    func toFlutterTransactions() -> [FlutterTransaction] {
        let flutterTransactions = map { transaction -> FlutterTransaction in
            let flutterTransaction = FlutterTransaction()
            flutterTransaction.id = transaction.id
            flutterTransaction.refId = transaction.refId
            flutterTransaction.orderCode = transaction.orderCode
            flutterTransaction.txnType = convertTransactionType(type: transaction.txnType)
            flutterTransaction.txnTypeText = transaction.txnTypeText
            flutterTransaction.txnDesc = transaction.txnDesc
            flutterTransaction.accountingType = convertAccoutingType(accounting: transaction.accountingType)
            flutterTransaction.status = convertStatus(status: transaction.status)
            flutterTransaction.point = NSNumber(value: transaction.point)
            flutterTransaction.createdAt = NSNumber(value: transaction.createdAt)
            flutterTransaction.createdBy = transaction.createdBy
            flutterTransaction.networkId = NSNumber(value: transaction.networkId)
            flutterTransaction.networkName = transaction.networkName
            flutterTransaction.merchantCode = transaction.merchantCode
            flutterTransaction.merchantName = transaction.merchantName
            flutterTransaction.terminalCode = transaction.terminalCode
            
            return flutterTransaction
            
        }
        
        return flutterTransactions
    }
}

private func convertStringTransactionType(transactionType: String) -> TransactionType {
    switch transactionType {
    case "01":
        return TransactionType.GRANT
    case "02":
        return TransactionType.SPEND
    case "03":
        return TransactionType.RETURN_GRANT
    case "04":
        return TransactionType.RETURN_SPEND
    default:
        return TransactionType.GRANT
    }
}

private func convertStatus(status: FlutterStatus) -> Int {
    switch status {
    case FlutterStatus.`init`:
        return 1
    case FlutterStatus.initFailure:
        return 2
    case FlutterStatus.noAccouning:
        return 4
    case FlutterStatus.accountingSuccess:
        return 4
    case FlutterStatus.accountingFailure:
        return 5
    case FlutterStatus.accouting:
        return 6
    default:
        return 4
    }
}

private func convertTransactionType(type: TransactionType) -> FlutterTransactionType {
    switch type {
    case TransactionType.GRANT:
        return FlutterTransactionType.grant
    case TransactionType.SPEND:
        return FlutterTransactionType.spend
    case TransactionType.RETURN_GRANT:
        return FlutterTransactionType.returnGrant
    case TransactionType.RETURN_SPEND:
        return FlutterTransactionType.returnSpend
    default:
        return FlutterTransactionType.grant
    }
}

private func convertFlutterCardType(card: LoyaltyCardType) -> FlutterCardType {
    switch card {
    case LoyaltyCardType.memberCard:
        return FlutterCardType.memberCard
    case LoyaltyCardType.qr:
        return FlutterCardType.qr
    }
}

extension NetworkConfig {
    func toFlutterNetworkConfig() -> FlutterNetworkConfig {
        let flutterNetworkConfig = FlutterNetworkConfig()
        flutterNetworkConfig.exchangeRate = convertIntToNSNumber(exchangeRate)
        flutterNetworkConfig.currencyImg = currencyImg
        flutterNetworkConfig.currencyName = currencyName
        flutterNetworkConfig.cardDefaultImg = cardDefaultImg
        flutterNetworkConfig.avatarDefaultImg = avatarDefaultImg
        flutterNetworkConfig.allowToGetMemberByPhone = convertBoolToNSNumber(allowGetMemberByPhone)
        flutterNetworkConfig.faqWebUrl = faqWebUrl
        flutterNetworkConfig.benefitWebUrl = benefitWebUrl
        flutterNetworkConfig.brandColor = brandColor
        flutterNetworkConfig.allowMerchantIssueMemberCard = convertBoolToNSNumber(allowMerchantIssueMemberCard)
        
        return flutterNetworkConfig
    }
}

extension Optional where Wrapped == LoyaltyPaging {
    func toFlutterPaging() -> FlutterPageData {
        let pageData = FlutterPageData()
        
        pageData.currentPage = convertIntToNSNumber(self?.totalItems)
        pageData.totalPage = convertIntToNSNumber(self?.totalPage)
        pageData.pageSize = convertIntToNSNumber(self?.pageSize)
        pageData.totalItems = convertIntToNSNumber(self?.totalItems)
        
        return pageData
    }
}
