import '../model/loyalty_configuration.dart';
import '../model/member_info.dart';
import '../model/membership_qr.dart';
import '../model/netword_config_result.dart';
import '../model/register_member_request.dart';
import '../model/status.dart';
import '../model/transaction_history.dart';
import '../model/transaction_history_request.dart';
import '../model/transaction_type.dart';
import 'loyalty_api.dart';

class ModelConverter {
  ModelConverter._();

  static LoyaltyConfiguration toModelLoyaltyConfiguration(
      FlutterLoyaltyConfiguration flutterLoyaltyConfiguration) {
    return LoyaltyConfiguration(
        linkMembershipUrl: flutterLoyaltyConfiguration.linkMembershipUrl ?? "",
        registerMemberUrl: flutterLoyaltyConfiguration.registerMemberUrl ?? "",
        allowLinkMembership: flutterLoyaltyConfiguration.allowLinkMembership ?? false,
        allowRegisterMembership: flutterLoyaltyConfiguration.allowRegisterMembership ?? false,
        membershipCodeFormat: flutterLoyaltyConfiguration.membershipCodeFormat ?? "");
  }

  static MemberInfo toModelMemberInfo(FlutterMemberInfo flutterMemberInfo) {
    return MemberInfo(
      memberQrCode: flutterMemberInfo.memberQrCode,
      memberId: flutterMemberInfo.memberId ?? "",
      phone: flutterMemberInfo.phone ?? "",
      memberCardId: flutterMemberInfo.memberCardId,
      name: flutterMemberInfo.name ?? "",
      isActive: flutterMemberInfo.isActive ?? false,
      point: flutterMemberInfo.point ?? 0,
      tierPoint: flutterMemberInfo.tierPoint,
      keepTierUntil: flutterMemberInfo.keepTierUntil,
      accumulationFrom: flutterMemberInfo.accumulationFrom,
      accumulationTo: flutterMemberInfo.accumulationTo,
      currentTierCode: flutterMemberInfo.currentTierCode,
      currentTierName: flutterMemberInfo.currentTierName,
      currentTierMinPoint: flutterMemberInfo.currentTierMinPoint,
      nextTierName: flutterMemberInfo.nextTierName,
      nextTierMinPoint: flutterMemberInfo.nextTierMinPoint,
      defaultCardType: flutterMemberInfo.defaultCardType.toCardType(),
    );
  }

  static EncryptedMemCode toModelEncryptedMemCode(FlutterMembershipQr membershipQr) {
    return EncryptedMemCode(
      code: membershipQr.code ?? "",
      issueAt: membershipQr.issueAt ?? 0,
      expireAt: membershipQr.expireAt ?? 0,
    );
  }

  static NetworkConfigResult toModelNetworkConfigResult(FlutterNetworkData networkData) {
    final networkInfo = Network(
      networkData.networkInfo?.id ?? "",
      networkData.networkInfo?.name ?? "",
    );
    final flutterTiers = networkData.networkTiers;
    final networkConfig = networkData.networkConfig.toNetworkConfig();

    final networkTiers = flutterTiers
        ?.map((e) => Tier(
              code: e?.code ?? "",
              name: e?.name ?? "",
              fromPoint: e?.fromPoint ?? 1,
              cardTieredImg: e?.cardTieredImg,
              tierIconImg: e?.tierIconImg,
            ))
        .toList();

    return NetworkConfigResult(
      networkInfo: networkInfo,
      networkConfig: networkConfig,
      networkTiers: networkTiers,
    );
  }

  static TransactionHistory toModelTransactionHistory(FlutterTransactionHistory result) {
    final pageData = PageData(
      currentPage: result.pageData?.currentPage ?? 0,
      totalPage: result.pageData?.totalPage ?? 0,
      pageSize: result.pageData?.pageSize ?? 0,
      totalItems: result.pageData?.totalItems ?? 0,
    );

    return TransactionHistory(
      transactions: result.transactions.toTxns() ?? [],
      pageData: pageData,
    );
  }

  static FlutterRegisterMemeberRequest toFlutterRegisterMemberRequest(
      RegisterMemeberRequest request) {
    return FlutterRegisterMemeberRequest(
      phone: request.phone,
      name: request.name,
      address: request.address,
      dateOfBirth: request.dateOfBirth,
      email: request.email,
      gender: request.gender.toFlutterGender(),
      idCardNumber: request.idCardNumber,
      idCitizenNumber: request.idCitizenNumber,
      passportNumber: request.passportNumber,
    );
  }

  static FlutterTransactionHistoryRequest toFlutterTransactionHistoryRequest(
      TransactionHistoryRequest request) {
    return FlutterTransactionHistoryRequest(
      merchantCode: request.merchantCode,
      fromTime: request.fromTime,
      toTime: request.toTime,
      txnTypes: request.txnTypes.toFlutterTxns(),
      status: request.status.toFlutterStatus(),
      page: request.page,
      pageSize: request.pageSize,
    );
  }
}

extension StatusX on Status? {
  FlutterStatus? toFlutterStatus() {
    switch (this) {
      case Status.init:
        return FlutterStatus.init;
      case Status.initFailure:
        return FlutterStatus.initFailure;
      case Status.noAccouning:
        return FlutterStatus.noAccouning;
      case Status.accountingSuccess:
        return FlutterStatus.accountingSuccess;
      case Status.accountingFailure:
        return FlutterStatus.accountingFailure;
      case Status.accouting:
        return FlutterStatus.accouting;
      default:
        return FlutterStatus.accountingSuccess;
    }
  }
}

extension FlutterCardTypeX on FlutterCardType? {
  CardType? toCardType() {
    if (this != null) {
      if (this == FlutterCardType.memberCard) {
        return CardType.memberCard;
      } else {
        return CardType.qr;
      }
    }

    return null;
  }
}

extension ListTransactionTypesX on List<TransactionType>? {
  List<String>? toFlutterTxns() {
    final txnTypes = this?.map((e) {
      String type;
      switch (e) {
        case TransactionType.grant:
          type = "01";
          break;
        case TransactionType.spend:
          type = "02";
          break;
        case TransactionType.returnGrant:
          type = "03";
          break;
        case TransactionType.returnSpend:
          type = "04";
          break;
        case TransactionType.initiate:
          type = "05";
          break;
        case TransactionType.initiatePromotion:
          type = "06";
          break;
        case TransactionType.issue:
          type = "07";
          break;
      }

      return type;
    });

    return txnTypes?.toList();
  }
}

extension GenderX on Gender? {
  FlutterGender toFlutterGender() {
    switch (this) {
      case Gender.male:
        return FlutterGender.male;
      case Gender.female:
        return FlutterGender.female;
      default:
        return FlutterGender.none;
    }
  }
}

extension FlutterNetworkConfigX on FlutterNetworkConfig? {
  NetworkConfig toNetworkConfig() {
    return NetworkConfig(
      exchangeRate: this?.exchangeRate ?? 0,
      currencyImg: this?.currencyImg ?? "",
      currencyName: this?.currencyName ?? "",
      cardDefaultImg: this?.cardDefaultImg,
      avatarDefaultImg: this?.avatarDefaultImg,
      allowToGetMemberByPhone: this?.allowToGetMemberByPhone,
      faqWebUrl: this?.faqWebUrl,
      benefitWebUrl: this?.benefitWebUrl,
      brandColor: this?.brandColor,
      allowMerchantIssueMemberCard: this?.allowMerchantIssueMemberCard,
    );
  }
}

extension ListFlutterTransactionsX on List<FlutterTransaction?>? {
  List<Transaction>? toTxns() {
    final transactions = this?.map((e) {
      TransactionType txnType;
      switch (e?.txnType) {
        case FlutterTransactionType.grant:
          txnType = TransactionType.grant;
          break;
        case FlutterTransactionType.spend:
          txnType = TransactionType.spend;
          break;
        case FlutterTransactionType.returnGrant:
          txnType = TransactionType.returnGrant;
          break;
        case FlutterTransactionType.returnSpend:
          txnType = TransactionType.returnSpend;
          break;
        case FlutterTransactionType.initiate:
          txnType = TransactionType.initiate;
          break;
        case FlutterTransactionType.initiatePromotion:
          txnType = TransactionType.initiatePromotion;
          break;
        case FlutterTransactionType.issue:
          txnType = TransactionType.issue;
          break;
        default:
          txnType = TransactionType.grant;
          break;
      }

      AccountingType accountingType;
      switch (e?.accountingType) {
        case FlutterAccountingType.debit:
          accountingType = AccountingType.debit;
          break;
        default:
          accountingType = AccountingType.credit;
          break;
      }

      Status status;
      switch (e?.status) {
        case FlutterStatus.initFailure:
          status = Status.initFailure;
          break;
        case FlutterStatus.noAccouning:
          status = Status.noAccouning;
          break;
        case FlutterStatus.accountingSuccess:
          status = Status.accountingSuccess;
          break;
        case FlutterStatus.accountingFailure:
          status = Status.accountingFailure;
          break;
        case FlutterStatus.accouting:
          status = Status.accouting;
          break;
        default:
          status = Status.accountingSuccess;
          break;
      }

      return Transaction(
        id: e?.id ?? "",
        refId: e?.refId,
        orderCode: e?.orderCode,
        txnType: txnType,
        txnTypeText: e?.txnTypeText ?? "",
        txnDesc: e?.txnDesc,
        accountingType: accountingType,
        status: status,
        point: e?.point ?? 0,
        createdAt: e?.createdAt ?? 0,
        createdBy: e?.createdBy ?? "",
        networkId: e?.networkId ?? 0,
        networkName: e?.networkName ?? "",
        merchantCode: e?.merchantCode,
        merchantName: e?.merchantName,
        terminalCode: e?.terminalCode,
        terminalName: e?.terminalName,
        terminalAddress: e?.terminalAddress,
      );
    });

    return transactions?.toList();
  }
}